<?php

namespace v4\wp\plugins\oauth;

use Exception;
use League\OAuth2\Client\Provider\AbstractProvider;
use League\OAuth2\Client\Provider\GenericProvider;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Token\AccessToken;
use League\OAuth2\Client\Token\AccessTokenInterface;
use Psr\Http\Message\ResponseInterface;

class Identity
{
    protected static ?Identity $instance = null;

    private GenericProvider $provider;

    private string $client_id;
    private string $client_secret;
    private string $redirect_uri;
    private array $scopes;
    private string $url;
    private string $url_auth;
    private string $url_token;
    private string $url_resource;
    private string $url_logout;
    private string $url_lost_password;
    private string $url_change_password;

    private string $lang;

    /** @var AccessTokenInterface $access_token */
    private AccessTokenInterface $access_token;

    public static function getInstance(): Identity
    {
        if (is_null(self::$instance)) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    /**
     * v4oAuthIdentity constructor.
     * @throws Exception
     */
    public function __construct()
    {
//        session_start();
        require_once __DIR__ . '/vendor/autoload.php';

        $_SESSION['V4OA']['PROVIDER'] = '1CRM';
        $v4oauth_option = get_option('v4_oauth');

        if (empty($v4oauth_option['client_id']) ||
            empty($v4oauth_option['client_secret']) ||
            empty($v4oauth_option['1crm_url'])
        ) {
            throw new Exception('oAuth not configured', 500);
        }

        $locale = get_locale();
        $lang = explode('_', $locale)[0];
        $this->lang = !empty($lang) ? $lang : 'de';
        if (defined('ICL_LANGUAGE_CODE')) {
            $this->lang = ICL_LANGUAGE_CODE;
        }

        $this->client_id = $v4oauth_option['client_id'];
        $this->client_secret = $v4oauth_option['client_secret'];

        $this->redirect_uri = rtrim(site_url(), '/') . '/';

        $this->scopes = [
            'basic',
            'portal'
        ]; // PROVIDER SPECIFIC: 'profile' is the minimum scope required to get the user's id from 1crm

        $login_server_url = trailingslashit($v4oauth_option['1crm_url']);
        if (empty($v4oauth_option['disable_auto_url_suffix'])) {
            $login_server_url .= "loginServer/index.php/";
        }

        $this->url = $login_server_url;

        $this->url_auth = $this->url . "authorize";
        $this->url_token = $this->url . "access_token";
        $this->url_resource = $this->url . "resource";
        $this->url_logout = $this->url . "logout";
        $this->url_lost_password = $this->url . "lostpassword";
        $this->url_change_password = $this->url . "changepassword";

        $this->provider = new GenericProvider([
            'clientId' => $this->client_id,    // The client ID assigned to you by the provider
            'clientSecret' => $this->client_secret,   // The client password assigned to you by the provider
            'redirectUri' => $this->redirect_uri,
            'urlAuthorize' => $this->url_auth,
            'urlAccessToken' => $this->url_token,
            'urlResourceOwnerDetails' => $this->url_resource,
            'scopes' => implode(' ', $this->scopes),
            'pkceMethod' => AbstractProvider::PKCE_METHOD_S256,
        ]);

        $this->validateTokenExpire();
    }

    private function createRedirectUrl(): string
    {
        /*
         * redirect URL
         */
        // try to obtain the redirect_url from the default login page:
        $redirect_url = !empty($_GET['redirect_to']) ? esc_url($_GET['redirect_to']) : '';

        // if no redirect_url was found, set it to the user's last page:
        if (!$redirect_url) {
            $redirect_url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") .
                "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
            $redirect_url = explode('?', $redirect_url);
            $redirect_url = $redirect_url[0];
        }

        //if still false get_home_url
        if (!$redirect_url) {
            $redirect_url = explode(
                '?',
                function_exists('wpml_get_home_url_filter') ? wpml_get_home_url_filter() : site_url()
            )[0];
        }

        return $redirect_url;
    }

    public function validateTokenExpire(): bool
    {
        $result = false;
        if (!empty($_SESSION['V4OA']['ACCESS_TOKEN']) && !empty($_SESSION['V4OA']['REFRESH_TOKEN'])) {
            $this->access_token = new AccessToken([
                'refresh_token' => $_SESSION['V4OA']['REFRESH_TOKEN'],
                'access_token' => $_SESSION['V4OA']['ACCESS_TOKEN'],
                'expires' => $_SESSION['V4OA']['EXPIRES'],
            ]);
            if ($this->access_token->hasExpired()) {
                $refresh_token = $this->access_token->getRefreshToken();
                try {
                    $this->access_token = $this->provider->getAccessToken('refresh_token', [
                        'refresh_token' => $refresh_token
                    ]);
                    $_SESSION['V4OA']['REFRESH_TOKEN'] = $this->access_token->getRefreshToken();
                    $_SESSION['V4OA']['ACCESS_TOKEN'] = $this->access_token->getToken();
                    $_SESSION['V4OA']['EXPIRES'] = $this->access_token->getExpires();
                    if ($this->access_token->hasExpired()) {
                        $this->unsetAccessToken();
                    } else {
                        $result = true;
                    }
                } catch (Exception $e) {
                    $this->unsetAccessToken();
                }
            } else {
                $result = true;
            }
        }
        return $result;
    }

    public function unsetAccessToken(): void
    {
        unset($this->access_token);
        unset($_SESSION['V4OA']['REFRESH_TOKEN']);
        unset($_SESSION['V4OA']['ACCESS_TOKEN']);
        unset($_SESSION['V4OA']['EXPIRES']);
    }

    /**
     * @throws Exception
     */
    public function auth()
    {
        if ($this->validateTokenExpire()) {
            return true;
        }

        /*
         * Process
         */
        //If we don't have an authorization code then get one
        if (!isset($_GET['code'])) {
            $redirect_url = $this->createRedirectUrl();

            // set the user's last page, so we can return that user there after they log in:
            $_SESSION['V4OA']['LAST_URL'] = $redirect_url;
            $_SESSION['V4OA']['LANG'] = $this->lang;


            /*
             * auth
             */
            // Fetch the authorization URL from the provider; this returns the
            // urlAuthorize option and generates and applies any necessary parameters
            // (e.g. state).
            $authorizationUrl = $this->provider->getAuthorizationUrl();
            $authorizationUrl .= '&lang=' . ($this->lang);

            //providers
            $provider = $_GET['connect'];
            if (in_array($provider, array_keys(settings::$providers))) {
                $authorizationUrl .= '&provider=' . $provider;
            }

            // Get the state generated for you and store it to the session.
            $_SESSION['V4OA']['OAUTH2STATE'] = $this->provider->getState();
            $_SESSION['V4OA']['OAUTH2PKCECODE'] = $this->provider->getPkceCode();

            // Redirect the user to the authorization URL.
            header('Location: ' . $authorizationUrl);
            exit;

            // Check given state against previously stored one to mitigate CSRF attack
        } elseif (empty($_GET['state']) ||
            (isset($_SESSION['V4OA']['OAUTH2STATE']) && $_GET['state'] !== $_SESSION['V4OA']['OAUTH2STATE'])
        ) {
            if (isset($_SESSION['V4OA']['OAUTH2STATE'])) {
                unset($_SESSION['V4OA']['OAUTH2STATE']);
            }

            $this->error('Invalid state');
        } else {
            try {
                // Optional, only required when PKCE is enabled.
                // Restore the PKCE code stored in the session.
                $this->provider->setPkceCode($_SESSION['V4OA']['OAUTH2PKCECODE']);

                // Try to get an access token using the authorization code grant.
                $this->access_token = $this->provider->getAccessToken('authorization_code', [
                    'code' => $_GET['code']
                ]);
                // We have an access token, which we may use in authenticated

                if ($this->access_token->hasExpired()) {
                    $this->access_token = $this->provider->getAccessToken('refresh_token', [
                        'refresh_token' => $this->access_token->getRefreshToken()
                    ]);
                }

                $_SESSION['V4OA']['REFRESH_TOKEN'] = $this->access_token->getRefreshToken();
                $_SESSION['V4OA']['ACCESS_TOKEN'] = $this->access_token->getToken();
                $_SESSION['V4OA']['EXPIRES'] = $this->access_token->getExpires();

                return true;
            } catch (IdentityProviderException $e) {
                $response_body = $e->getResponseBody();
                error_log('PHP Error: v4OAuthIdentity (' . $e->getCode() . ') ' . json_encode($response_body));
                // Failed to get the access token or user details.
                $this->error($e->getMessage(), $e->getCode());
            }
        }
    }

    public function lostPassword(): void
    {
        $redirect_url = $this->createRedirectUrl();

        // set the user's last page so, we can return that user there after they log in:
        $_SESSION['V4OA']['LAST_URL'] = $redirect_url;
        $_SESSION['V4OA']['LANG'] = $this->lang;

        //lost password
        if (isset($_GET['lost_password'])) {
            // Redirect the user to the password change URL.
            header('Location: ' . $this->url_lost_password . '?redirect_uri=' . $redirect_url);
            exit;
        }
    }

    public function changePassword(): void
    {
        if (isset($_GET['result'])) {
            if (!empty($_GET['result'])) {
                $result_str = 'Password changed';
            } else {
                $result_str = 'Password change failed';
                $_SESSION['V4OA']['ERROR'] = true;
            }
            $_SESSION['V4OA']['RESULT'] = __($result_str, 'v4-oauth');
        } else {
            $redirect_url = $this->createRedirectUrl();

            // set the user's last page, so we can return that user there after they log in:
            $_SESSION['V4OA']['LAST_URL'] = $redirect_url;
            $_SESSION['V4OA']['LANG'] = $this->lang;

            //change password
            if (isset($_GET['change_password'])) {
                // Redirect the user to the password change URL.
                header('Location: ' . $this->url_change_password . '?redirect_uri=' . $redirect_url);
                exit;
            }
        }
    }

    /**
     * @return array
     * @throws IdentityProviderException
     */
    public function apiMe(): array
    {
        // The provider provides a way to get an authenticated API request for
        // the service, using the access token; it returns an object conforming
        // to Psr\Http\Message\RequestInterface.
        $params = array(
            'lang' => $this->lang
        ); // TODO umbau zu accepted headers

        //return parsed response
        $me = $this->api('me' . '?' . http_build_query($params), $params);
        return json_decode($me->getBody()->getContents(), true);
    }

    /**
     * @depecated use api() instead
     * @throws IdentityProviderException
     */
    public function apiGet($endpoint, $options = []): ResponseInterface
    {
        return $this->api($endpoint, $options);
    }

    /**
     * @throws IdentityProviderException
     */
    public function api($endpoint, $options = [], $method = 'GET'): ResponseInterface
    {
        if (!$this->validateTokenExpire()) {
            $err_msg = __('You are not logged in.');
            throw new IdentityProviderException($err_msg, 401, $err_msg);
        }

        if (!isset($options['oauth_user_id'])) {
            $options['oauth_user_id'] = get_user_meta(get_current_user_id(), 'crm_id', true);
        }

        $url = $this->url . 'api/' . $endpoint;

        $request = $this->provider->getAuthenticatedRequest(
            $method,
            $url,
            $this->access_token,
            $options
        );

        $response = $this->provider->getResponse($request);
        $status_code = $response->getStatusCode();

        if ($status_code == 200) {
            return $response;
        } else {
            error_log('PHP Error: v4OAuthIdentity api (' . $status_code . ') ');
            $err_msg = 'Es ist ein Fehler aufgetreten. ' .
                'Versuchen Sie es noch einmal oder kontaktieren Sie den Administrator dieser Webseite.';
            throw new IdentityProviderException($err_msg, $status_code, $response->getBody()->getContents());
        }
    }

    public function logout(): void
    {
        # AUTHENTICATION FLOW #
        // the oauth 2.0 authentication flow will start in this script and make several calls to the third-party
        // authentication provider which in turn will make callbacks to this script that we continue to handle until
        // the login completes with a success or failure:

        // try to obtain the redirect_url from the default login page:
        $redirect_url = !empty($_GET['redirect_to']) ? esc_url($_GET['redirect_to']) : '';
        // if no redirect_url was found, set it to the user's last page:
        if (!$redirect_url) {
            $redirect_url = strtok($_SERVER['HTTP_REFERER'], "?");
        }
        // set the user's last page, so we can return that user there after they log in:
        unset($_SESSION['V4OA']['LAST_URL']);
        $this->unsetAccessToken();

        $params = array(
            'redirect_uri' => $redirect_url
        );
        $url_params = http_build_query($params);
        $url = $this->url_logout . '?' . $url_params;
        header('Location: ' . $url, true, 301);
        die();
    }

    /**
     * @param $msg
     * @param int $code
     * @throws Exception
     */
    private function error($msg, int $code = 400): void
    {
        throw new Exception($msg, $code);
    }
}
