<?php

namespace v4\wp\plugins\oauth;

class Settings
{
    /**
     * Holds the values to be used in the fields callbacks
     */
    private array $options = [];

    public static array $providers = [
        'google' => 'Google'
    ];

    /**
     * Start up
     */
    public function __construct()
    {
        add_action('admin_menu', array($this, 'addPluginPage'), 20);
        add_action('admin_init', array($this, 'pageInit'));
    }

    /**
     * Add options page
     */
    public function addPluginPage()
    {
        // create new top-level menu
        if (!empty($GLOBALS['admin_page_hooks']['wp1crm-extensions'])) {
            add_submenu_page(
                'wp1crm-extensions',
                'v4 oAuth',
                'v4 oAuth',
                'manage_options',
                'v4-OAuth',
                [$this, 'createAdminPage']
            );
        } else {
            add_options_page(
                'v4-OAuth Options',
                'v4-OAuth',
                'manage_options',
                'v4-OAuth',
                [$this, 'createAdminPage']
            );
        }
    }

    /**
     * Options page callback
     */
    public function createAdminPage()
    {
        // Set class property
        $this->options = get_option('v4_oauth', []);
        $action = 'options.php';
        ?>
        <div class="wrap">
            <h1>v4 oAuth Settings</h1>
            <form method="post" action="<?= $action ?>">
                <?php
                // This prints out all hidden setting fields
                settings_fields('v4_oauth_group');
                do_settings_sections('v4-oauth-admin');
                submit_button();
                ?>
            </form>
        </div>
        <?php
    }

    /**
     * Register and add settings
     */
    public function pageInit()
    {
        register_setting(
            'v4_oauth_group', // Option group
            'v4_oauth', // Option name
            [$this, 'sanitize'] // Sanitize
        );

        add_settings_section(
            'v4oauth_base_setting', // ID
            'v4 oAuth Settings', // Title
            [$this, 'printSectionInfo'], // Callback
            'v4-oauth-admin' // Page
        );

        //Fields
        add_settings_field(
            '1crm_url',
            '1CRM URL',
            [$this, 'callback1crmUrl'],
            'v4-oauth-admin',
            'v4oauth_base_setting'
        );
        add_settings_field(
            'disable_auto_url_suffix',
            'Disable auto URL suffix',
            [$this, 'callbackDisableAutoUrlSuffix'],
            'v4-oauth-admin',
            'v4oauth_base_setting'
        );
        add_settings_field(
            'client_id',
            'Client ID',
            [$this, 'callbackClientId'],
            'v4-oauth-admin',
            'v4oauth_base_setting'
        );
        add_settings_field(
            'client_secret',
            'Client Secret',
            [$this, 'callbackClientSecret'],
            'v4-oauth-admin',
            'v4oauth_base_setting'
        );
        add_settings_field(
            'providers',
            'External oAuth Providers',
            [$this, 'callbackProviders'],
            'v4-oauth-admin',
            'v4oauth_base_setting'
        );
        add_settings_field(
            'formidable_register_field_email',
            'Formidable Register Field Key E-Mail',
            [$this, 'callbackFormidableRegisterFieldEmail'],
            'v4-oauth-admin',
            'v4oauth_base_setting'
        );
//        add_settings_field(
//            'sync_events',
//            'Sync Events',
//            array($this, 'callback_sync_events'),
//            'v4-oauth-admin',
//            'v4oauth_base_setting'
//        );
    }

    /**
     * Sanitize each setting field as needed
     *
     * @param array $input Contains all settings fields as array keys
     * @return array $new_input
     */
    public function sanitize(array $input): array
    {
        $new_input = array();

        //TODO check checkboxes

        if (isset($input['1crm_url'])) {
            $new_input['1crm_url'] = filter_var($input['1crm_url'], FILTER_SANITIZE_URL);
        }
        $new_input['disable_auto_url_suffix'] = $input['disable_auto_url_suffix'] ? 1 : 0;
        if (isset($input['client_id'])) {
            $new_input['client_id'] = $input['client_id']; //no sanitize for id
        }
        if (isset($input['client_secret'])) {
            $new_input['client_secret'] = $input['client_secret']; //no sanitize for secret
        }

        $new_input['formidable_register_field_email'] = $input['formidable_register_field_email'] ?? "email1";

        //checkboxes
        $new_input['sync_events'] = (isset($input['sync_events']) && $input['sync_events'] == 'on');
        $new_input['sync_event_bookings'] = (isset($input['sync_event_bookings']) &&
            $input['sync_event_bookings'] == 'on');
        $new_input['use_legacy_v3'] = (isset($input['use_legacy_v3']) && $input['use_legacy_v3'] == 'on');

        //select
        if (isset($input['providers'])) {
            $new_input['providers'] = $input['providers'];
        }

        return $new_input;
    }

    /**
     * Print the Section text
     */
    public function printSectionInfo()
    {
        print 'Enter your settings below:';
    }

    /**
     * Get the settings option array and print one of its values
     */
    public function callback1crmUrl()
    {
        printf(
            '<input type="text" id="1crm_url" name="v4_oauth[1crm_url]" value="%s" />',
            isset($this->options['1crm_url']) ? esc_attr($this->options['1crm_url']) : ''
        );
    }

    public function callbackDisableAutoUrlSuffix()
    {
        $checked = isset($this->options['disable_auto_url_suffix']) && $this->options['disable_auto_url_suffix'] == 1 ?
            'checked="checked"' : '';
        echo "<input type='hidden' id='disable_auto_url_suffix' name='v4_oauth[disable_auto_url_suffix]' " .
            "value='0' />";
        echo "<input type='checkbox' id='disable_auto_url_suffix' name='v4_oauth[disable_auto_url_suffix]' " .
            "value='1' $checked />";
        echo "<p class='description'>disables auto suffix /loginServer/index.php</p>";
    }

    public function callbackClientId()
    {
        printf(
            '<input type="text" id="client_id" name="v4_oauth[client_id]" value="%s" />',
            isset($this->options['client_id']) ? esc_attr($this->options['client_id']) : ''
        );
    }

    public function callbackClientSecret()
    {
        printf(
            '<input type="text" id="client_secret" name="v4_oauth[client_secret]" value="%s" />',
            isset($this->options['client_secret']) ? esc_attr($this->options['client_secret']) : ''
        );
    }

    public function callbackProviders()
    {
        $active_providers = $this->options['providers'] ?? [];
        ?>
        <label>
            <select id="providers" name="v4_oauth[providers][]" multiple>
                <?php
                foreach (self::$providers as $k => $v) {
                    $selected = in_array($k, $active_providers) ? 'selected' : '';
                    ?>
                    <option <?= $selected ?> value="<?= $k ?>"><?= $v ?></option>
                    <?php
                }
                ?>
            </select>
        </label>
        <p class="description">has to be activated in 1CRM</p>
        <?php
    }

    public function callbackFormidableRegisterFieldEmail()
    {
        printf(
            '<input type="text" id="formidable_register_field_email" ' .
            'name="v4_oauth[formidable_register_field_email]" value="%s" placeholder="email1" />',
            isset($this->options['formidable_register_field_email']) ?
                esc_attr($this->options['formidable_register_field_email']) : ''
        );
    }

/*    public function callback_sync_events()
    {
        $checked = $this->options['sync_events'] ? 'checked="checked"' : '';
        ?>
        <input type="checkbox" id="sync_events" name="v4_oauth[sync_events]" <?= $checked ?> />
        <?php
    }
*/
}